@@ -1,10 +1,7 @@
 import atexit
-import base64
 import json
-import os
 import re
 from collections import namedtuple
-from pathlib import Path
 
 import requests
 import sys
@@ -45,7 +42,7 @@ class LogicHubAPI:
     __credentials = {}
     __session_cookie = None
 
-    def __init__(self, hostname, api_key=None, username=None, password=None, verify_ssl=True, cache_seconds=None, **kwargs):
+    def __init__(self, hostname, api_key=None, username=None, password=None, verify_ssl=True, cache_seconds=None, default_timeout=None, **kwargs):
         # First store key variables and then determine whether they are valid
         self.__api_key = api_key.strip() if api_key else None
         self.__username = username.strip() if username else None
@@ -59,6 +56,9 @@ class LogicHubAPI:
 
         if not self.log:
             self.log = Logger()
+        if default_timeout:
+            self.http_timeout_default = int(default_timeout)
+        self._http_timeout_login = min(self._http_timeout_login, self.http_timeout_default)
         self.kwargs = kwargs
 
         if isinstance(verify_ssl, bool):
@@ -302,28 +302,6 @@ class LogicHubAPI:
             self.log.debug("LogicHubAPI class exiting...")
             self.logout()
 
-    def __get_event_types_v1(self, limit):
-        # Not sure when this one stopped working, but it worked somewhere in at least m66 and didn't work any more in m70
-        params = {"limit": limit}
-        response = self._http_request(url=self.url.event_types, params=params)
-        results = response.json()
-        if results.get("result"):
-            results = results["result"]
-        if results.get("data"):
-            results = results["data"]
-        return results
-
-    def __get_event_types_v2(self, limit):
-        params = {"libraryView": "all"}
-        body = {"filters": [], "offset": 0, "pageSize": limit, "sortColumn": "lastUpdated", "sortOrder": "DESC"}
-        response = self._http_request(method="POST", url=self.url.event_types, params=params, body=body)
-        results = response.json()
-        if results.get("result"):
-            results = results["result"]
-        while isinstance(results, dict) and results.get('data') is not None:
-            results = results["data"]
-        return results
-
     # ToDo Revisit and finish this
     # def get_alerts(self, advanced_filter=None, limit=100):
     #     limit = int(limit) if limit is not None else 100_000
@@ -349,17 +327,32 @@ class LogicHubAPI:
     #     return response.json()
 
     def get_batch_results_by_id(self, batch_id: int, limit=1000, offset=0):
+        """
+
+        :param batch_id:
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :param offset: Used for pagination if you want to pull in chunks, this sets the page number to pull
+        :return:
+        """
         limit = int(limit or 1000)
         offset = int(offset or 0)
         params = {"fields": "*", "pageSize": limit, "after": offset, "cachedOnly": "true"}
         response = self._http_request(url=self.url.batch_results_by_id.format(int(batch_id)), params=params)
         return response.json()
 
-    def get_batches_by_stream_id(self, stream_id, limit=25, offset=0, statuses=None, exclude_empty_results=False):
-        if limit == -1:
-            limit = 999999999
+    def get_batches_by_stream_id(self, stream_id, limit=None, offset=0, statuses=None, exclude_empty_results=False):
+        """
+
+        :param stream_id:
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :param offset: Used for pagination if you want to pull in chunks, this sets the page number to pull
+        :param statuses:
+        :param exclude_empty_results:
+        :return:
+        """
+        limit = int(limit if limit and limit > 0 else 999999999)
         params = {
-            "pageSize": int(limit or 25),
+            "pageSize": limit,
             "after": int(offset or 0),
         }
         body = {"status": statuses or [], "excludeBatchesWithZeroEvents": exclude_empty_results}
@@ -426,62 +419,14 @@ class LogicHubAPI:
             raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing custom lists")
         return results
 
-    # ToDo shouldn't this be LIST instead of GET?
-    def get_custom_lists(self, search_text: str = None, filters: list = None, limit: int = None, offset: int = None, verify_results=True):
-        """
-
-        :param search_text: Optional: Part or all of the custom list name to filter by
-        :param filters: Optional: Advanced search, e.g. [{"searchText":"<case name>"}]
-        :param limit: Optional: Limit the number of results
-        :param offset: Optional: if using pagination, provide the entry number to fetch (default is 0)
-        :param verify_results: Optional: set to false to return response JSON without verifying that the format is as expected
-        :return:
-        """
-        if limit:
-            assert isinstance(limit, int) or int(limit)
-        if offset:
-            assert isinstance(offset, int) or int(offset)
-        limit = int(limit or 10_000)
-        offset = int(offset or 0)
-        filters = filters or []
-        if search_text:
-            filters.append({"searchText": search_text})
-
-        response = self._http_request(
-            method="POST",
-            url=self.url.custom_lists,
-            params={"libraryView": "all"},
-            body={
-                "filters": filters,
-                "offset": offset,
-                "pageSize": limit,
-                "sortColumn": "name",
-                "sortOrder": "ASC"
-            },
-            headers={'Content-Type': 'application/json'},
-        )
-        results = response.json()
-        if verify_results:
-            try:
-                _ = results["result"]["data"]
-            except Exception:
-                raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing custom lists")
-        return results
-
-    # ToDo shouldn't this be LIST instead of GET?
-    def get_event_types(self, limit=25):
-        limit = int(limit or 25)
-        if self.version < 70:
-            return self.__get_event_types_v1(limit=limit)
-        return self.__get_event_types_v2(limit=limit)
-
-    # ToDo shouldn't this be LIST instead of GET?
-    def get_integrations(self):
-        response = self._http_request(method="GET", url=self.url.integrations)
+    # ToDo Token auth not supported as of 2022-03-09 (m92)
+    def get_dashboard_data(self, dashboard_id):
+        response = self._http_request(url=self.url.dashboard_data.format(dashboard_id))
         return response.json()
 
+    # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
     def get_rule_set_by_name(self, name):
-        rule_sets = self.get_rule_sets()
+        rule_sets = self.list_rule_sets()
         rule_set = [x for x in rule_sets if x['name'] == name]
         if not rule_set:
             raise exceptions.RuleSetNotFound(f"No rule set found matching name: {name}")
@@ -489,22 +434,6 @@ class LogicHubAPI:
         rule_set['rules'] = self.get_rules_for_rule_set(rule_set['id'])
         return rule_set
 
-    # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
-    # ToDo shouldn't this be LIST instead of GET?
-    def get_rule_sets(self, limit=25):
-        limit = int(limit or 25)
-        params = {
-            "fields": "name,isPublic",
-            "pageSize": limit,
-        }
-        response = self._http_request(url=self.url.rule_sets, params=params)
-        results = response.json()
-        try:
-            results = results["result"]["data"]
-        except Exception:
-            raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing rule sets")
-        return results
-
     # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
     def get_rules_for_rule_set(self, rule_set):
         rule_set = helpers.sanitize_input_rule_set_id(rule_set)
@@ -521,7 +450,7 @@ class LogicHubAPI:
 
     def get_stream_by_id(self, stream_id: int):
         headers = {"Accept": "application/json"}
-        # ToDo Revisit this. I better approach might be:
+        # ToDo Revisit this. A better approach might be:
         # try:
         #     response = self._http_request(url=self.url.stream_by_id.format(stream_id), headers=headers)
         #     return response.json()
@@ -536,15 +465,6 @@ class LogicHubAPI:
                 raise exceptions.StreamNotFound(stream_id)
         return response.json()
 
-    # ToDo shouldn't this be LIST instead of GET?
-    def get_stream_states(self, stream_ids: list):
-        new_stream_id_list = []
-        for s in stream_ids:
-            new_stream_id_list.append(f'stream-{s}')
-        body = {"streams": new_stream_id_list}
-        response = self._http_request(method="POST", url=self.url.stream_states, body=body)
-        return response.json()
-
     def get_version_info(self):
         self.log.debug("Fetching LogicHub version")
         try:
@@ -566,11 +486,6 @@ class LogicHubAPI:
         response = self._http_request(method="GET", url=self.url.case_status_workflow_by_id.format(workflow_id))
         return response.json()
 
-    # ToDo shouldn't this be LIST instead of GET?
-    def get_workflows(self):
-        response = self._http_request(method="GET", url=self.url.case_status_list_workflows)
-        return response.json()
-
     # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
     # ToDo Create an action method for this, and update the beta integration action to use it
     def add_scoring_rule(self, rule_set, field_mappings, score: float or str):
@@ -597,13 +512,20 @@ class LogicHubAPI:
         response = self._http_request(url=self.url.flow_export.format(playbook_id), test_response=test_response)
         return response.json()
 
-    def execute_command(self, command_payload, limit=25):
+    def execute_command(self, command_payload, limit=None):
+        """
+
+        :param command_payload:
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :return:
+        """
+        limit = int(limit if limit and limit > 0 else 999999999)
         response = self._http_request(
             url=self.url.command_execute,
             method="POST",
             body=command_payload,
             test_response=False,
-            params={"pageSize": int(limit or 25)}
+            params={"pageSize": limit}
         )
         try:
             result_dict = response.json()
@@ -652,24 +574,29 @@ class LogicHubAPI:
             self.log.fatal(message)
         return response.json()
 
-    def list_playbooks(self, limit=None):
-        limit = int(limit or 99999)
-        body = {"filters": [], "offset": 0, "pageSize": limit, "sortColumn": "name", "sortOrder": "ASC"}
-        response = self._http_request(
-            url=self.url.playbooks_list,
-            method="POST",
-            params={"pageSize": limit},
-            body=body
-        )
-        return response.json()
+    def list_baselines(self, limit=None, offset=0):
+        """
+        List all Baselines
 
-    def list_baselines(self):
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :param offset: Used for pagination if you want to pull in chunks, this sets the page number to pull
+        :return:
+        """
+        limit = int(limit if limit and limit > 0 else 999999999)
         params = {"libraryView": "all"}
-        body = {"filters": [], "offset": 0, "pageSize": 9999, "sortColumn": "lastUpdated", "sortOrder": "DESC"}
+        body = {"filters": [], "offset": offset or 0, "pageSize": limit, "sortColumn": "lastUpdated", "sortOrder": "DESC"}
         response = self._http_request(method="POST", url=self.url.baselines, params=params, body=body)
         return response.json()
 
-    def list_connections(self, limit=None, filters=None, offset=0):
+    def list_connections(self, filters=None, limit=None, offset=0):
+        """
+        List all connections
+
+        :param filters: Optional: Advanced search
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :param offset: Used for pagination if you want to pull in chunks, this sets the page number to pull
+        :return:
+        """
         limit = int(limit if limit and int(limit) and limit >= 1 else 99999)
         params = {"libraryView": "all"}
         headers = {"Content-Type": "application/json"}
@@ -677,16 +604,109 @@ class LogicHubAPI:
         response = self._http_request(method="POST", url=self.url.connections, headers=headers, params=params, body=body)
         return response.json()
 
+    def list_custom_lists(self, search_text: str = None, filters: list = None, limit: int = None, offset: int = None, verify_results=True):
+        """
+
+        :param search_text: Optional: Part or all of the custom list name to filter by
+        :param filters: Optional: Advanced search, e.g. [{"searchText":"<case name>"}]
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :param offset: Used for pagination if you want to pull in chunks, this sets the page number to pull
+        :param verify_results: Optional: set to false to return response JSON without verifying that the format is as expected
+        :return:
+        """
+        limit = int(limit if limit and limit > 0 else 999999999)
+        offset = int(offset or 0)
+        filters = filters or []
+        if search_text:
+            filters.append({"searchText": search_text})
+
+        response = self._http_request(
+            method="POST",
+            url=self.url.custom_lists,
+            params={"libraryView": "all"},
+            body={
+                "filters": filters,
+                "offset": offset,
+                "pageSize": limit,
+                "sortColumn": "name",
+                "sortOrder": "ASC"
+            },
+            headers={'Content-Type': 'application/json'},
+        )
+        results = response.json()
+        if verify_results:
+            try:
+                _ = results["result"]["data"]
+            except Exception:
+                raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing custom lists")
+        return results
+
+    # ToDo Token auth not supported as of 2022-03-09 (m92)
+    def list_dashboards(self):
+        response = self._http_request(url=self.url.dashboards)
+        return response.json()
+
     def list_dashboards_with_widgets(self):
         response = self._http_request(url=self.url.dashboards_and_widgets)
         return response.json()
 
+    def __list_event_types_v1(self, limit=None):
+        """
+
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :return:
+        """
+        # Not sure when this one stopped working, but it worked somewhere in at least m66 and didn't work any more in m70
+        limit = int(limit if limit and limit > 0 else 999999999)
+        params = {"limit": limit}
+        response = self._http_request(url=self.url.event_types, params=params)
+        results = response.json()
+        if results.get("result"):
+            results = results["result"]
+        if results.get("data"):
+            results = results["data"]
+        return results
+
+    def __list_event_types_v2(self, limit=None, offset=0):
+        """
+
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :param offset: Used for pagination if you want to pull in chunks, this sets the page number to pull
+        :return:
+        """
+        limit = int(limit if limit and limit > 0 else 999999999)
+        params = {"libraryView": "all"}
+        # Confirm the right term for "name" as the sort column and change this sort order
+        body = {"filters": [], "pageSize": limit, "offset": offset or 0, "sortColumn": "lastUpdated", "sortOrder": "DESC"}
+        response = self._http_request(method="POST", url=self.url.event_types, params=params, body=body)
+        results = response.json()
+        if results.get("result"):
+            results = results["result"]
+        while isinstance(results, dict) and results.get('data') is not None:
+            results = results["data"]
+        return results
+
+    def list_event_types(self, limit=None):
+        """
+
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :return:
+        """
+        limit = int(limit if limit and limit > 0 else 999999999)
+        if self.version < 70:
+            return self.__list_event_types_v1(limit=limit)
+        return self.__list_event_types_v2(limit=limit)
+
     def list_fields(self, params: dict = None, **kwargs):
         params = params or {"systemFields": "true", "pageSize": 9999, "after": 0}
         response = self._http_request(method="GET", url=self.url.fields, params=params, **kwargs)
         self.fields = response.json()
         return response.json()
 
+    def list_integrations(self):
+        response = self._http_request(method="GET", url=self.url.integrations)
+        return response.json()
+
     def list_ml_models(self):
         response = self._http_request(
             method="GET",
@@ -703,11 +723,19 @@ class LogicHubAPI:
         )
         return response.json()
 
-    def list_notebooks(self, limit=None, search_string=None):
+    def list_notebooks(self, limit=None, offset=0, search_string=None):
+        """
+        List all Notebooks
+
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :param offset: Used for pagination if you want to pull in chunks, this sets the page number to pull
+        :param search_string:
+        :return:
+        """
         limit = limit if limit and isinstance(limit, int) else 99999
         body = {
             "filters": [],
-            "offset": 0,
+            "offset": offset or 0,
             "pageSize": limit,
             "sortColumn": "name",
             "sortOrder": "ASC",
@@ -726,7 +754,63 @@ class LogicHubAPI:
         self.notebooks = response["result"]["data"]["data"] = helpers.sort_notebook_objects_by_id(response["result"]["data"]["data"])
         return response
 
-    def list_streams(self, search_text: str = None, filters: list = None, limit: int = 25, offset: int = 0):
+    def list_playbooks(self, limit=None, offset=0):
+        """
+        List all Playbooks
+
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :param offset: Used for pagination if you want to pull in chunks, this sets the page number to pull
+        :return:
+        """
+        limit = int(limit if limit and limit > 0 else 999999999)
+        body = {"filters": [], "offset": offset or 0, "pageSize": limit, "sortColumn": "name", "sortOrder": "ASC"}
+        response = self._http_request(
+            url=self.url.playbooks_list,
+            method="POST",
+            params={"pageSize": limit},
+            body=body
+        )
+        return response.json()
+
+    # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
+    def list_rule_sets(self, limit=None):
+        """
+
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :return:
+        """
+        limit = int(limit if limit and limit > 0 else 999999999)
+        params = {
+            "fields": "name,isPublic",
+            "pageSize": limit,
+        }
+        response = self._http_request(url=self.url.rule_sets, params=params)
+        results = response.json()
+        try:
+            results = results["result"]["data"]
+        except Exception:
+            raise exceptions.UnexpectedOutput("API response does not match the expected schema for listing rule sets")
+        return results
+
+    def list_stream_states(self, stream_ids: list):
+        new_stream_id_list = []
+        for s in stream_ids:
+            new_stream_id_list.append(f'stream-{s}')
+        body = {"streams": new_stream_id_list}
+        response = self._http_request(method="POST", url=self.url.stream_states, body=body)
+        return response.json()
+
+    def list_streams(self, search_text: str = None, filters: list = None, limit: int = None, offset: int = 0):
+        """
+        List all streams
+
+        :param search_text:
+        :param filters:
+        :param limit: None by default, although the LogicHub UI defaults to 25
+        :param offset: Used for pagination if you want to pull in chunks, this sets the page number to pull
+        :return:
+        """
+        limit = int(limit if limit and limit > 0 else 999999999)
         params = {"libraryView": "all"}
         headers = {"Content-Type": "application/json"}
 
@@ -736,7 +820,7 @@ class LogicHubAPI:
 
         body_dict = {
             "filters": filters,
-            "offset": offset,
+            "offset": offset or 0,
             "pageSize": limit,
             "sortColumn": "name",
             "sortOrder": "ASC"
@@ -745,6 +829,10 @@ class LogicHubAPI:
         response = self._http_request(method="POST", url=self.url.streams, body=body_dict, headers=headers, params=params)
         return response.json()
 
+    def list_workflows(self):
+        response = self._http_request(method="GET", url=self.url.case_status_list_workflows)
+        return response.json()
+
     def reprocess_batch(self, batch_id):
         """
         Reprocess an errored, canceled, or stale batch using its batch ID
