@@ -105,7 +105,7 @@ class Actions:
             _dict = _dict[field]
         return True
 
-    def execute_command(self, command_name, input_dict, reformat=True, result_limit=25):
+    def execute_command(self, command_name, input_dict, reformat=True, result_limit=None):
         response = self.__api.execute_command({"command": command_name, "parameterValues": input_dict, "limit": result_limit})
         if not reformat:
             return response
@@ -156,6 +156,13 @@ class Actions:
     def get_case_prefix(self):
         return self.__api.case_prefix
 
+    # ToDo Token auth not supported as of 2022-03-09 (m92)
+    def get_dashboard_data(self, dashboard_id):
+        result = self.__api.get_dashboard_data(dashboard_id.strip())
+        _ = self._result_dict_has_schema(result, "result", "widgets", raise_errors=True, action_description="get dashboard data")
+        result = result['result']['widgets']
+        return result
+
     def get_notebooks_attached_to_case(self, case_id):
         response = self.__api.case_list_attached_notebooks(case_id)
         _ = self._result_dict_has_schema(response, "result", action_description="get notebooks from case", raise_errors=True)
@@ -186,7 +193,7 @@ class Actions:
         result = result["result"]["data"]
         baselines = result['data']
         stream_ids = [int(b['id']['id']) for b in baselines]
-        stream_state_response = self.__api.get_stream_states(stream_ids=stream_ids)
+        stream_state_response = self.__api.list_stream_states(stream_ids=stream_ids)
         _ = self._result_dict_has_schema(stream_state_response, "result", "streams", raise_errors=True, action_description="fetch stream states")
         state_map = {_stream['streamId']: _stream['status'] for _stream in stream_state_response['result']['streams']}
         for n in range(len(baselines)):
@@ -195,8 +202,8 @@ class Actions:
             baselines[n]['baseline_config_status'] = state_map.get(f"stream-{b['id']['id']}")
         return result
 
-    def list_connections(self, add_status=False):
-        result = self.__api.list_connections()
+    def list_connections(self, filters=None, add_status=False):
+        result = self.__api.list_connections(filters=filters)
         _ = self._result_dict_has_schema(result, "result", "data", action_description="list connections", raise_errors=True)
         result = result["result"]["data"]
         if not add_status:
@@ -215,7 +222,7 @@ class Actions:
         return result
 
     def list_custom_lists(self, search_text: str = None, filters: list = None, limit: int = None, offset: int = None):
-        results = self.__api.get_custom_lists(search_text=search_text, filters=filters, limit=limit, offset=offset, verify_results=False)
+        results = self.__api.list_custom_lists(search_text=search_text, filters=filters, limit=limit, offset=offset, verify_results=False)
         warnings = []
         try:
             results = results["result"]["data"]
@@ -223,9 +230,16 @@ class Actions:
             warnings.append("API response does not match the expected schema for listing custom lists")
         return results, warnings
 
+    # ToDo Token auth not supported as of 2022-03-09 (m92)
+    def list_dashboards(self):
+        result = self.__api.list_dashboards()
+        _ = self._result_dict_has_schema(result, "result", raise_errors=True, action_description="list dashboards")
+        result = result['result']
+        return result
+
     def list_dashboards_with_widgets(self, include_not_imported=True):
         result = self.__api.list_dashboards_with_widgets()
-        _ = self._result_dict_has_schema(result, "result", "objects", raise_errors=True, action_description="list fields")
+        _ = self._result_dict_has_schema(result, "result", "objects", raise_errors=True, action_description="list dashboards with widgets")
         result = result['result']['objects']
         if not include_not_imported:
             result = [r for r in result if r['metadata']["contentRepoStatus"] != "Global"]
@@ -247,7 +261,7 @@ class Actions:
         assert filter_type in ["equals", "contains"], f"Invalid filter type \"{filter_type}\""
         assert isinstance(name_filter, str), "name_filter must be a string and cannot be None"
 
-        _response = self.__api.get_integrations()
+        _response = self.__api.list_integrations()
 
         # _query = _response.pop("query")
         _result = _response.pop("result")
@@ -293,7 +307,7 @@ class Actions:
         _ = self._result_dict_has_schema(response, "result", "data", "data")
         return response["result"]["data"]["data"]
 
-    def list_streams(self, search_text: str = None, filters: list = None, limit: int = 25, offset: int = 0):
+    def list_streams(self, search_text: str = None, filters: list = None, limit: int = None, offset: int = 0):
         result = self.__api.list_streams(search_text=search_text, filters=filters, limit=limit, offset=offset)
         _ = self._result_dict_has_schema(result, "result", "data", "data", raise_errors=True, action_description="list streams")
         return result["result"]["data"]["data"]
@@ -309,7 +323,7 @@ class Actions:
         return result["result"]
 
     def list_workflows(self):
-        result = self.__api.get_workflows()
+        result = self.__api.list_workflows()
         _ = self._result_dict_has_schema(result, "result", "data", action_description="list workflows", raise_errors=True)
         return result["result"]["data"]
 
@@ -377,7 +391,7 @@ class Actions:
 class LogicHub:
     verify_ssl = True
 
-    def __init__(self, hostname, api_key=None, username=None, password=None, verify_ssl=True, cache_seconds=None, verify_api_auth=True, logger: Logger = None, log_level=None, **kwargs):
+    def __init__(self, hostname, api_key=None, username=None, password=None, verify_ssl=True, cache_seconds=None, verify_api_auth=True, logger: Logger = None, log_level=None, default_timeout=None, **kwargs):
         # If the LogicHubAPI class object has not been given a logger by the time this class is instantiated, set one for it
         LogicHubAPI.log = LogicHubAPI.log or logger or Logger()
         if log_level:
@@ -386,7 +400,7 @@ class LogicHub:
         LogicHubAPI.verify_ssl = self.verify_ssl
 
         self.kwargs = kwargs
-        self.api = LogicHubAPI(hostname=hostname, api_key=api_key, username=username, password=password, verify_ssl=verify_ssl, cache_seconds=cache_seconds, **kwargs)
+        self.api = LogicHubAPI(hostname=hostname, api_key=api_key, username=username, password=password, verify_ssl=verify_ssl, cache_seconds=cache_seconds, default_timeout=default_timeout, **kwargs)
         self.actions = Actions(self.api)
         if verify_api_auth:
             _ = self._verify_api_auth()
